const char *solid_first_fs =
  "//////////////////////////////////////////////////////////////////////\n"
  "//\n"
  "// First-stage screen-space fragment shader for the solid pipeline\n"
  "//\n"
  "// It offers ambient occlusion and edge detection capabilities.\n"
  "//\n"
  "//////////////////////////////////////////////////////////////////////\n"
  "\n"
  "#version 120\n"
  "\n"
  "//\n"
  "// Input\n"
  "//\n"
  "\n"
  "// texture coordinates\n"
  "varying vec2 UV;\n"
  "\n"
  "//\n"
  "// Uniforms\n"
  "//\n"
  "\n"
  "// RGB rendered texture\n"
  "uniform sampler2D inRGBTex;\n"
  "// Depth rendered texture\n"
  "uniform sampler2D inDepthTex;\n"
  "// 1.0 if enabled, 0.0 if disabled\n"
  "uniform float inAoEnabled;\n"
  "// Shadow strength for SSAO\n"
  "uniform float inAoStrength;\n"
  "// 1.0 if enabled, 0.0 if disabled\n"
  "uniform float inEdStrength;\n"
  "// Rendering surface dimensions, in pixels\n"
  "uniform float width, height;\n"
  "\n"
  "vec3 getNormalAt(vec2 normalUV)\n"
  "{\n"
  "  float xpos = texture2D(inDepthTex, normalUV + vec2(1.0 / width, 0.0)).x;\n"
  "  float xneg = texture2D(inDepthTex, normalUV - vec2(1.0 / width, 0.0)).x;\n"
  "  float ypos = texture2D(inDepthTex, normalUV + vec2(0.0, 1.0 / height)).x;\n"
  "  float yneg = texture2D(inDepthTex, normalUV - vec2(0.0, 1.0 / height)).x;\n"
  "  float xdelta = xpos - xneg;\n"
  "  float ydelta = ypos - yneg;\n"
  "  vec3 r = vec3(xdelta, ydelta, 1.0 / width + 1.0 / height);\n"
  "  return normalize(r);\n"
  "}\n"
  "\n"
  "vec3 getNormalNear(vec2 normalUV)\n"
  "{\n"
  "  float cent = texture2D(inDepthTex, normalUV).x;\n"
  "  float xpos = texture2D(inDepthTex, normalUV + vec2(1.0 / width, 0.0)).x;\n"
  "  float xneg = texture2D(inDepthTex, normalUV - vec2(1.0 / width, 0.0)).x;\n"
  "  float ypos = texture2D(inDepthTex, normalUV + vec2(0.0, 1.0 / height)).x;\n"
  "  float yneg = texture2D(inDepthTex, normalUV - vec2(0.0, 1.0 / height)).x;\n"
  "  float xposdelta = xpos - cent;\n"
  "  float xnegdelta = cent - xneg;\n"
  "  float yposdelta = ypos - cent;\n"
  "  float ynegdelta = cent - yneg;\n"
  "  float xdelta = abs(xposdelta) > abs(xnegdelta) ? xnegdelta : xposdelta;\n"
  "  float ydelta = abs(yposdelta) > abs(ynegdelta) ? ynegdelta : yposdelta;\n"
  "  vec3 r = vec3(xdelta, ydelta, 0.5 / width + 0.5 / height);\n"
  "  return normalize(r);\n"
  "}\n"
  "\n"
  "float lerp(float a, float b, float f)\n"
  "{\n"
  "    return a + f * (b - a);\n"
  "}\n"
  "\n"
  "const vec2 SSAOkernel[16] = vec2[16](\n"
  "  vec2(0.072170, 0.081556),\n"
  "  vec2(-0.035126, 0.056701),\n"
  "  vec2(-0.034186, -0.083598),\n"
  "  vec2(-0.056102, -0.009235),\n"
  "  vec2(0.017487, -0.099822),\n"
  "  vec2(0.071065, 0.015921),\n"
  "  vec2(0.040950, 0.079834),\n"
  "  vec2(-0.087751, 0.065326),\n"
  "  vec2(0.061108, -0.025829),\n"
  "  vec2(0.081262, -0.025854),\n"
  "  vec2(-0.063816, 0.083857),\n"
  "  vec2(0.043747, -0.068586),\n"
  "  vec2(-0.089848, 0.049046),\n"
  "  vec2(-0.065370, 0.058761),\n"
  "  vec2(0.099581, -0.089322),\n"
  "  vec2(-0.032077, -0.042826)\n"
  ");\n"
  "\n"
  "float computeSSAOLuminosity(vec3 normal)\n"
  "{\n"
  "  float totalOcclusion = 0.0;\n"
  "  float depth = texture2D(inDepthTex, UV).x;\n"
  "  float A = (width * UV.x + 10 * height * UV.y) * 2.0 * 3.14159265358979 * 5.0 / 16.0;\n"
  "  float S = sin(A);\n"
  "  float C = cos(A);\n"
  "  mat2 rotation = mat2(\n"
  "    C, -S,\n"
  "    S, C\n"
  "  );\n"
  "  for (int i = 0; i < 16; i++) {\n"
  "    vec2 samplePoint = rotation * SSAOkernel[i];\n"
  "    float occluderDepth = texture2D(inDepthTex, UV + samplePoint).x;\n"
  "    vec3 occluder = vec3(samplePoint.xy, depth - occluderDepth);\n"
  "    float d = length(occluder);\n"
  "    float occlusion = max(0.0, dot(normal, occluder)) * (1.0 / (1.0 + d));\n"
  "    totalOcclusion += occlusion;\n"
  "  }\n"
  "  \n"
  "  return max(0.0, 1.2 - inAoStrength * totalOcclusion);\n"
  "}\n"
  "\n"
  "float computeEdgeLuminosity(vec3 normal)\n"
  "{\n"
  "  return max(0.0, pow(normal.z - 0.1, 1.0 / 3.0));\n"
  "}\n"
  "\n"
  "void main()\n"
  "{\n"
  "  float luminosity = 1.0;\n"
  "  luminosity *= max(1.2 * (1.0 - inAoEnabled), computeSSAOLuminosity(getNormalNear(UV)));\n"
  "  luminosity *= max(1.0 - inEdStrength, computeEdgeLuminosity(getNormalAt(UV)));\n"
  "\n"
  "  vec4 color = texture2D(inRGBTex, UV);\n"
  "  gl_FragColor = vec4(color.xyz * luminosity, color.w);\n"
  "  gl_FragDepth = texture2D(inDepthTex, UV).x;\n"
  "}\n"
  "\n";
